package daba;

import daba.db.Administrator;
import daba.db.Person;
import daba.db.Student;
import daba.db.Teacher;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.UserTransaction;

@WebServlet(name = "ListOfServlet", urlPatterns = {"/listof"})
public class ListOfServlet extends HttpServlet {
                
    @PersistenceContext
    private EntityManager entityManager;
    
    @Resource
    private UserTransaction userTra;
    
    private void renderPersonHeader(PrintWriter out)
    {
        out.println("<th>action</th><th>first name</th><th>last name</th>");
    }
    
    private void renderPerson(PrintWriter out, Person p)
    {
        out.println("<td>" + p.getFIRSTNAME() + "</td><td>" + p.getLASTNAME() + "</td>");
    }

    private void renderStudentTab(PrintWriter out)
    {
        out.println("<table><thead><tr>");
        renderPersonHeader(out);
        out.println("<th>study direction</th></thead><tbody>");
        Query q = entityManager.createQuery("SELECT s FROM Student s");
        @SuppressWarnings("unchecked")
        List<Student> studentList = (List<Student>) q.getResultList();
        for (Student student : studentList)
        {
            out.println("<tr><td><a href=\"delete?type=student&id=" + student.getPerson().getId() + "\">delete</a></td>");
            renderPerson(out, student.getPerson());                
            out.println("<td>" + student.getDirection() + "</td>");
            out.println("</tr>\n");
        }
        out.println("</tbody></table>");
    }

    private void renderTeacherTab(PrintWriter out)
    {
        out.println("<table><thead><tr>");
        renderPersonHeader(out);
        out.println("<th>department</th></thead><tbody>");
        Query q = entityManager.createQuery("SELECT t FROM Teacher t");
        @SuppressWarnings("unchecked")
        List<Teacher> teacherList = (List<Teacher>)q.getResultList();
        for (Teacher teacher : teacherList)
        {
            out.println("<tr><td><a href=\"delete?type=teacher&id=" + teacher.getPerson().getId() + "\">delete</a></td>");
            renderPerson(out, teacher.getPerson());                
            out.println("<td>" + teacher.getDepartment() + "</td>");
            out.println("</tr>\n");
        }
        out.println("</tbody></table>");
    }
                 
    private void renderAdministratorTab(PrintWriter out)
    {
        out.println("<table><thead><tr>");
        renderPersonHeader(out);
        out.println("<th>position</th></thead><tbody>");
        Query q = entityManager.createQuery("SELECT a FROM Administrator a");
        @SuppressWarnings("unchecked")
        List<Administrator> administratorList = (List<Administrator>) q.getResultList();
        for (Administrator administrator : administratorList)
        {
            out.println("<tr><td><a href=\"delete?type=administrator&id=" + administrator.getPerson().getId() + "\">delete</a></td>");
            renderPerson(out, administrator.getPerson());                
            out.println("<td>" + administrator.getPosition() + "</td>");
            out.println("</tr>\n");
        }
        out.println("</tbody></table>");
    }

    private void printHeaderAndIntro(PrintWriter out)
    {
        out.println("<!DOCTYPE html>");
        out.println("<html>");
        out.println("<head>");
        out.println("<title>List of records</title>");            
        out.println("</head>");
        out.println("<body>");
        out.println("<h1>List of records</h1>");
    }
    
    private void printFooter(PrintWriter out)
    {
        out.println("</body>");
        out.println("</html>");
    }
    
    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code>
     * methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setContentType("text/html;charset=UTF-8");
        try (PrintWriter out = response.getWriter()) {
           
            printHeaderAndIntro(out);
            
            String tab = request.getParameter("tab");
            
            if (tab.equals("student")) renderStudentTab(out);
            else if (tab.equals("teacher")) renderTeacherTab(out);
            else if (tab.equals("administrator")) renderAdministratorTab(out);

            printFooter(out);
        }
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
